/*
** Module   : ImageWidget.java
** Abstract : server side image widget implementation
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20141208 Created initial version.
** 002 EVL 20150115 Rework for image support implementation.
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side image widget implementation.
 */
@LegacyResource(resource = LegacyResource.IMAGE)
public class ImageWidget
extends BaseEntity<ImageConfig>
{
   /**
    * Default constructor.
    */
   public ImageWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public ImageWidget(boolean dynamic)
   {
      super(dynamic, new ImageConfig());
   }
   
   /**
    * Returns the name of the image loaded by LOAD-IMAGE. Corresponds Progress
    * IMAGE attribute
    *
    * @return the name of the image loaded by LOAD-IMAGE. 
    */
   public character getImage()
   {
      return new character(config.image);
   }

   /**
    * Set button image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImage(String name)
   {
      loadImage(name);
   }
   
   /**
    * Set image tooltip.
    * 
    * @param   text
    *          Tooltip text
    */
   public void setTooltip(String text)
   {
      config.tooltip = text;
      pushScreenDefinition();
   }

   /**
    * Get image tooltip.
    * 
    * @return  image tooltip.
    */
   public character getTooltip()
   {
      return new character(config.tooltip);
   }
   
   /**
    * Set CONVERT-3D-COLORS option.
    * 
    * @param   flag
    *          New value for the option.
    */
   public void setConvert3D(boolean flag)
   {
      config.convert3D = flag;
      pushScreenDefinition();
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImage(String name)
   {
      return new logical(loadImageInt(name));
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImage(character name)
   {
      return name != null && !name.isUnknown() ? loadImage(name.toStringMessage())
                                               : new logical(false);
   }

   /**
    * Load image for widget.  Internal version.
    * 
    * @param    imageSrc
    *           Image file name to load.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   private boolean loadImageInt(String imageSrc)
   {
      // save current values
      String imageNameSaved = config.image;
      
      // this must be done before sending image bytes because the image should be on client side
      if (frame != null)
      {
         config.image = imageSrc;
         frame.pushScreenDefinition(true);
      }
      
      // logical terminal will try to load image from application jar, then ask the
      // client to load from local file system in cas of failure
      boolean result = LogicalTerminal.setImage(
                          config.id.getId(), ButtonConfig.IMAGE_IMG, imageSrc);
      
      // if we are here - there is no image to load we need to undo image name change
      if (!result && frame != null)
      {
         config.image = imageNameSaved;
         frame.pushScreenDefinition(true);
      }
      
      return result;
   }
}
